/*----------------------------------------------------------------------------

  UNIVERZITA TOMASE BATI VE ZLINE
  FAKULTA TECHNOLOGICKA
  INSTITUT INFORMACNICH TECHNOLOGII

  Linearni deterministicke fraktaly

  Cantorovo diskontinium	D = 0.63
  Kochova vlocka			D = 1.26
  Sierpinskeho trojuhelnik	D = 1.59
  Sierpinskeho ctverec  	D = 1.89

  
  ---
  Jan Stalmach, leto 2004
  stalmachjan@volny.cz

  Verze pro Visual Studio .NET 2003

----------------------------------------------------------------------------*/

#include <math.h>
#include "glut.h"	// knihovna glut pro praci s okny


// vertikalni posun u zobrazovani iteraci Cantorova diskontinia
#define POSUN 0.05

// definovani datoveho typu bod = (X,Y)
typedef GLfloat bod[2];

// aktualni hodnota iterace
int iterace = 0; 


/*---------------------------------------------------------
 Cantorovo diskontinium
-----------------------------------------------------------*/
void Cantor(bod begin, bod end, int n)
{
	if(n>=0)   
	{   
		// zacatek
		bod pointA = {begin[0],begin[1]-POSUN};  
		// 2/3   
		bod pointB = {(2.0 * begin[0]+end[0])/3.0,((2.0 * begin[1]+end[1])/3.0)-POSUN };       
		// 1/3 delky    
		bod pointC = {(begin[0]+2.0 * end[0])/3.0,((begin[1]+2.0 *end[1])/3.0)-POSUN};    
		// koncovy bod	
		bod pointD = {end[0],end[1]-POSUN}; 
		
		Cantor(pointA, pointB,n-1);
		Cantor(pointC, pointD,n-1);

		glColor3f(1.0,1.0,1.0); 
		glBegin(GL_LINES);            
			glVertex2f(begin[0],begin[1]);       
			glVertex2f(end[0],end[1]);       
		glEnd(); 
	}
}

/*---------------------------------------------------------
 Fraktalni Kochova vlocka
-----------------------------------------------------------*/
void Kochova(bod begin, bod end, int n)
{
	if(n<=0)  
	{    
		glColor3f(1.0,1.0,0.0);    
		glBegin(GL_LINES);            
			glVertex2f(begin[0],begin[1]);       
			glVertex2f(end[0],end[1]);       
		glEnd();  
	}  else  
	{     
		// zacatek
		bod pointA = {begin[0],begin[1]};  
		// 2/3   
		bod pointB = {(2.0 * begin[0]+end[0])/3.0,(2.0 * begin[1]+end[1])/3.0 };   
		// bod nad 1/2    
		bod pointC = {(begin[0]+end[0])/2.0 - sqrt(3.0)/6.0 *(end[1]-begin[1]),
			(begin[1]+end[1])/2.0 + sqrt(3.0)/6.0 *(end[0]-begin[0])};    
		// 1/3 delky    
		bod pointD = {(begin[0]+2.0 * end[0])/3.0,(begin[1]+2.0 *end[1])/3.0 };    
		// koncovy bod	
		bod pointE = {end[0],end[1]}; 
		
		// a do b  
		Kochova(pointA, pointB, n-1);   	  
		// b do c  
		Kochova(pointB, pointC, n-1);   
		// c do d  
		Kochova(pointC, pointD, n-1);   
		// d do e    
		Kochova(pointD, pointE, n-1);   
		
	}
}

/*----------------------------------------------------
 Kochova vlocka
------------------------------------------------------*/
void Kochova_vlocka(bod point1, bod point2, bod point3)
{     
	Kochova(point1, point2, iterace);   
	Kochova(point2, point3, iterace);   
	Kochova(point3, point1, iterace);
} 



/*---------------------------------------------------------
  Sierpinskeho trojuhelnik
-----------------------------------------------------------*/
void Sierpinski(bod A, bod B, bod C, int n)
{
	if(n<=0)  
	{    
		glColor3f(1.0,0.0,0.0);   
		glBegin(GL_TRIANGLES);            
		glVertex2f(A[0],A[1]);       
		glVertex2f(B[0],B[1]);
		glVertex2f(C[0],C[1]);
		glEnd(); 		
	}  else  
	{     
		bod pointA = {A[0],A[1]};
		bod pointB = {(A[0]+B[0])/2.0 ,(A[1]+B[1])/2.0 };
		bod pointC = {(A[0]+C[0])/2.0 ,(A[1]+C[1])/2.0 }; 
		bod pointD = {(B[0]+C[0])/2.0 ,(B[1]+C[1])/2.0 };
		bod pointE = {B[0],B[1]};
		bod pointF = {C[0],C[1]}; 
		
		Sierpinski(pointA, pointB, pointC, n-1); 
		Sierpinski(pointB, pointE, pointD, n-1);
		Sierpinski(pointC, pointD, pointF, n-1); 
	}
}


/*---------------------------------------------------------
  Sierpinskeho ctverec
-----------------------------------------------------------*/
void Sierpinski_ctverec(bod A, bod B, bod C, bod D, int n)
{
	if(n<=0)  
	{    
		glColor3f(0.0,1.0,0.0);   
		glBegin(GL_QUADS);            
		glVertex2f(A[0],A[1]);       
		glVertex2f(B[0],B[1]);
		glVertex2f(C[0],C[1]);
		glVertex2f(D[0],D[1]);
		glEnd(); 		
	}  else  
	{     
		bod pointA = {A[0],A[1]};
		bod pointB = {(A[0]*2.0+B[0])/3.0 ,(A[1]*2.0+B[1])/3.0 };
		bod pointC = {(A[0]*2.0+C[0])/3.0 ,(A[1]*2.0+C[1])/3.0 }; 
		bod pointD = {(A[0]*2.0+D[0])/3.0 ,(A[1]*2.0+D[1])/3.0 };

		bod pointE = {(A[0]+B[0]*2.0)/3.0 ,(A[1]+B[1]*2.0)/3.0 };
		bod pointF = {(B[0]*2.0+D[0])/3.0 ,(B[1]*2.0+D[1])/3.0 }; 

		bod pointG = {B[0],B[1]};
		bod pointH = {(B[0]*2.0+C[0])/3.0 ,(B[1]*2.0+C[1])/3.0 };

		bod pointI = {(D[0]*2.0+B[0])/3.0 ,(D[1]*2.0+B[1])/3.0 };
		bod pointJ = {(A[0]+D[0]*2.0)/3.0 ,(A[1]+D[1]*2.0)/3.0 };

		bod pointK = {(C[0]*2.0+B[0])/3.0 ,(C[1]*2.0+B[1])/3.0 };
		bod pointL = {(A[0]+C[0]*2.0)/3.0 ,(A[1]+C[1]*2.0)/3.0 };

		bod pointM = {(D[0]*2.0+C[0])/3.0 ,(D[1]*2.0+C[1])/3.0 };
		bod pointN = {D[0],D[1]};

		bod pointO = {(C[0]*2.0+D[0])/3.0 ,(C[1]*2.0+D[1])/3.0 };

		bod pointP = {C[0],C[1]};
			
		Sierpinski_ctverec(pointA, pointB, pointC, pointD, n-1);
		Sierpinski_ctverec(pointB, pointE, pointF, pointC, n-1);
		Sierpinski_ctverec(pointE, pointG, pointH, pointF, n-1);
		Sierpinski_ctverec(pointD, pointC, pointI, pointJ, n-1);
		Sierpinski_ctverec(pointF, pointH, pointK, pointL, n-1);
		Sierpinski_ctverec(pointJ, pointI, pointM, pointN, n-1);
		Sierpinski_ctverec(pointI, pointL, pointO, pointM, n-1);
		Sierpinski_ctverec(pointL, pointK, pointP, pointO, n-1);
	}
}


/*---------------------------------
 Inicializuje vlastnosti OpenGL
-----------------------------------*/
void Inicializuj_OpenGL(void)
{
	// barva kterou se maze colorbuffer
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	// Vyplnuj polygony
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
}


/*--------------------------------------------
 Prekresli okno do spravnych pomeru stran
 ----------------------------------------------*/
void Reshape(int w, int h)
{
	glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);   // prepne se na projekcni matici
    glLoadIdentity();
	// vypocita projekci
	glOrtho(-1.2, 1.2, -0.6,1.5, -10.0, 10.0);
    glMatrixMode(GL_MODELVIEW);	   // prepne se zpatky na modelovaci matici
    glLoadIdentity (); 
}


/*----------------------------- 
  Funkce cte tisknute klavesy
--------------------------------*/
void Klavesnice(unsigned char key, int x, int y)
{
   switch (key) {

   case 27: {           // pri ESC skonci aplikaci
       
				exit(0); 	  
	    	    break;
		   };

   case '+': {          
       
				if (iterace<10) iterace++; 	  
	    	    break;
		   };

   case '-': {          
       
				if (iterace>0) iterace--; 	  
	    	    break;
		   };

	 default : break;
      };
}


/*------------------------------------
  Funkce zajistijici renderovani sceny
--------------------------------------*/
void Render(void)
{
	// vycisti backbuffer
	glClear(GL_COLOR_BUFFER_BIT); 

	// Kochova vlocka
	bod A = {-1,0};
	bod B = {0,1.44};
	bod C = {1,0};
	Kochova_vlocka(A,B,C);

	// Cantorovo diskontinium
	bod X = {-0.5,0.5};
	bod Y = {0.5,0.5};
	Cantor(X,Y,iterace);

	// Sierpinskeho trojuhelnik
	bod P = {0,1.25};
	bod Q = {-0.5,0.55};
	bod R = {0.5,0.55};
	Sierpinski(P,Q,R,iterace);

	// Sierpinskeho ctverec
	bod I = {-0.2,0.2};
	bod J = {0.2,0.2};
	bod K = {0.2,-0.2};
	bod L = {-0.2,-0.2};
	Sierpinski_ctverec(I,J,K,L,iterace);

	// prekresleni bufferu
    glutPostRedisplay();

	// prohodi videobuffery
	glutSwapBuffers();
}


/*----------------------------
 Hlavni volana fce main()
-----------------------------*/
void main(int argc, char** argv)
{
   // inicializuj glut
   glutInit(&argc, argv);
   // pouzij doublebuffering a RGB model
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
   // nastav velikost okna aplikace
   glutInitWindowSize(800, 600);
   // nastav pozici na monitoru
   glutInitWindowPosition(50, 50);
   // vytvor renderovaci okno
   glutCreateWindow("Linearni deterministicke fraktaly: (+) zvysi iteraci (-) snizi iteraci -> Jan Stalmach FT-UTB Zlin 2004 <-");
   // nastav potrebne parametry OpenGL
   Inicializuj_OpenGL();
   // call-back fce
   glutDisplayFunc(Render); 
   glutReshapeFunc(Reshape);
   glutKeyboardFunc(Klavesnice); 
   // hlavni smycka
   glutMainLoop(); 
} 